--
-- FST990 + Ev for FS25
-- @author: _Near
-- @changelog:
-- 		- 1.0.0.0
--			- Initial Mod 
--

-- @class EvToggle Main Mod script
EvToggle = {};

function EvToggle.prerequisitesPresent(specializations)
	return true
end

function EvToggle.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, 'toggleEv', EvToggle.toggleEv)
    SpecializationUtil.registerFunction(vehicleType, 'haveEvToggle', EvToggle.haveEvToggle)
    SpecializationUtil.registerFunction(vehicleType, 'setDischargeableDiesel', EvToggle.setDischargeableDiesel)
    SpecializationUtil.registerFunction(vehicleType, 'setDischargeableEv', EvToggle.setDischargeableEv)
    SpecializationUtil.registerFunction(vehicleType, 'haveEvConfiguration', EvToggle.haveEvConfiguration)
end

function EvToggle:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
	local spec = self.spec_EvToggle
	if not spec.modInitialized then
		return
	end
	
	if g_dedicatedServerInfo ~= nil then
		return
	end
	if spec.eventList == nil then
		spec.eventList = {}
	end

    if self:getIsActiveForInput() or (self.getIscontrolled ~= nil and self:getIsControlled()) then
        local _, eventID = g_inputBinding:registerActionEvent(InputAction.EV_TOGGLE, self, EvToggle.actionCallback, true, true, false, true)

        spec.eventList[InputAction.EV_TOGGLE] = eventID

        if g_inputBinding ~= nil and g_inputBinding.events ~= nil and g_inputBinding.events[eventID] ~= nil then
            g_inputBinding:setActionEventTextPriority(eventID, GS_PRIO_VERY_HIGH)
            g_inputBinding:setActionEventTextVisibility(eventID, self:haveEvToggle())
        end
    end

	if self:haveEvConfiguration() and spec.isEvMode then
		self:setDischargeableEv()
	else
		self:setDischargeableDiesel()
	end
end

function EvToggle.registerEventListeners(vehicleType)
	for _, value in pairs({ 'onLoad', 'onUpdate', 'onRegisterActionEvents', 'toggleEv' }) do
		SpecializationUtil.registerEventListener(vehicleType, value, EvToggle)
	end
end

function EvToggle:onLoad(savegame)
	self.spec_EvToggle = {}

	local spec = self.spec_EvToggle
	
	spec.modInitialized = true
	spec.eventList = {}
	spec.isEvMode = false
end

function EvToggle:onUpdate(dt, vehicle)
	local spec = self.spec_EvToggle
	
	if not spec.modInitialized then
		return
	end

	if self.isClient then
        if self:getIsActiveForInput() or (self.getIscontrolled ~= nil and self:getIsControlled()) then
			if spec.eventList ~= nil and g_dedicatedServerInfo == nil then
                for actionName,eventID in pairs(spec.eventList) do
                    if actionName == InputAction.EV_TOGGLE then
                        g_inputBinding:setActionEventActive(eventID, true)
                        g_inputBinding:setActionEventText(eventID, spec.isEvMode and g_i18n:getText('EV_TOGGLE_ON') or g_i18n:getText('EV_TOGGLE_OFF'))
                        g_inputBinding:setActionEventTextVisibility(eventID, self:haveEvToggle())
                    end
                end
            end
		end
	end
end

-- Function callback when event keys are pressed
-- @param string actionName = Action name triggered
-- @param number keyStatus = The key status cam be 1 or 0
function EvToggle:actionCallback(actionName, keyStatus)
	local spec = self.spec_EvToggle
	
	if not spec.modInitialized then
		return
	end

	if keyStatus == 1 then
		if actionName == 'EV_TOGGLE' then
			self:toggleEv()
		end
	end
end

-- Function to get if the vehicle have the ev toggle
-- @return boolean
function EvToggle:haveEvToggle()
    return self:getFillUnitExists(1) and self:getFillUnitExists(2)
end

-- Function to check if vehicle configuration is Ev
-- @return boolean
function EvToggle:haveEvConfiguration()
	local spec = self.spec_EvToggle

	if not spec.modInitialized then
		return false
	end

    if self:haveEvToggle() then
        return true
    end

    local haveEvConfig = false

    for index, _ in pairs(self:getFillUnitSupportedFillTypes(1)) do
		if tostring(index) == '126' then
            haveEvConfig = true
        end
    end

    return haveEvConfig
end

-- Function to configure the discharge node to use Ev
function EvToggle:setDischargeableEv()
	local spec = self.spec_dischargeable

	self.spec_dischargeable.currentDischargeNode.fillUnitIndex = 2
    self.spec_dischargeable.currentDischargeNode.emptySpeed = 1
end

-- Function to configure the discharge node to use Diesel
function EvToggle:setDischargeableDiesel()
	local spec = self.spec_dischargeable

	self.spec_dischargeable.currentDischargeNode.fillUnitIndex = 1
    self.spec_dischargeable.currentDischargeNode.emptySpeed = 20
end

-- Function to toggle the Ev mode and update the vehicle configuration
function EvToggle:toggleEv()
	local spec = self.spec_EvToggle
	
	if not spec.modInitialized then
		return
	end

    if self:haveEvToggle() == false then
        self:setDischargeableDiesel()
		return
	end

	spec.isEvMode = not spec.isEvMode

    if spec.isEvMode == true then
        self:setDischargeableEv()
    else
        self:setDischargeableDiesel()
    end
end
